@@ -2,7 +2,6 @@ package cli
 
 import (
 	"context"
-	"encoding/json"
 	"fmt"
 	"log"
 	"net/http"
@@ -14,9 +13,9 @@ import (
 	"syscall"
 	"time"
 
-	"github.com/99designs/aws-vault/v6/iso8601"
-	"github.com/99designs/aws-vault/v6/server"
-	"github.com/99designs/aws-vault/v6/vault"
+	"github.com/99designs/aws-vault/v7/iso8601"
+	"github.com/99designs/aws-vault/v7/server"
+	"github.com/99designs/aws-vault/v7/vault"
 	"github.com/99designs/keyring"
 	"github.com/alecthomas/kingpin"
 	"github.com/aws/aws-sdk-go-v2/aws"
@@ -29,24 +28,25 @@ type ExecCommandInput struct {
 	StartEc2Server   bool
 	StartEcsServer   bool
 	Lazy             bool
-	CredentialHelper bool
-	Config           vault.Config
+	JSONDeprecated   bool
+	Config           vault.ProfileConfig
 	SessionDuration  time.Duration
 	NoSession        bool
 	UseStdout        bool
+	ShowHelpMessages bool
 }
 
 func (input ExecCommandInput) validate() error {
 	if input.StartEc2Server && input.StartEcsServer {
-		return fmt.Errorf("Can't use --server with --ecs-server")
+		return fmt.Errorf("Can't use --ec2-server with --ecs-server")
 	}
-	if input.StartEc2Server && input.CredentialHelper {
-		return fmt.Errorf("Can't use --server with --json")
+	if input.StartEc2Server && input.JSONDeprecated {
+		return fmt.Errorf("Can't use --ec2-server with --json")
 	}
 	if input.StartEc2Server && input.NoSession {
-		return fmt.Errorf("Can't use --server with --no-session")
+		return fmt.Errorf("Can't use --ec2-server with --no-session")
 	}
-	if input.StartEcsServer && input.CredentialHelper {
+	if input.StartEcsServer && input.JSONDeprecated {
 		return fmt.Errorf("Can't use --ecs-server with --json")
 	}
 	if input.StartEcsServer && input.NoSession {
@@ -62,10 +62,14 @@ func (input ExecCommandInput) validate() error {
 	return nil
 }
 
+func hasBackgroundServer(input ExecCommandInput) bool {
+	return input.StartEcsServer || input.StartEc2Server
+}
+
 func ConfigureExecCommand(app *kingpin.Application, a *AwsVault) {
 	input := ExecCommandInput{}
 
-	cmd := app.Command("exec", "Executes a command with AWS credentials in the environment")
+	cmd := app.Command("exec", "Execute a command with AWS credentials.")
 
 	cmd.Flag("duration", "Duration of the temporary or assume-role session. Defaults to 1h").
 		Short('d').
@@ -84,11 +88,12 @@ func ConfigureExecCommand(app *kingpin.Application, a *AwsVault) {
 
 	cmd.Flag("json", "Output credentials in JSON that can be used by credential_process").
 		Short('j').
-		BoolVar(&input.CredentialHelper)
+		Hidden().
+		BoolVar(&input.JSONDeprecated)
 
-	cmd.Flag("server", "Alias for --ec2-server. Run a EC2 metadata server in the background for credentials").
+	cmd.Flag("server", "Alias for --ecs-server").
 		Short('s').
-		BoolVar(&input.StartEc2Server)
+		BoolVar(&input.StartEcsServer)
 
 	cmd.Flag("ec2-server", "Run a EC2 metadata server in the background for credentials").
 		BoolVar(&input.StartEc2Server)
@@ -108,17 +113,17 @@ func ConfigureExecCommand(app *kingpin.Application, a *AwsVault) {
 		StringVar(&input.ProfileName)
 
 	cmd.Arg("cmd", "Command to execute, defaults to $SHELL").
-		Default(os.Getenv("SHELL")).
 		StringVar(&input.Command)
 
 	cmd.Arg("args", "Command arguments").
 		StringsVar(&input.Args)
 
 	cmd.Action(func(c *kingpin.ParseContext) (err error) {
-		input.Config.MfaPromptMethod = a.PromptDriver
+		input.Config.MfaPromptMethod = a.PromptDriver(hasBackgroundServer(input))
 		input.Config.NonChainedGetSessionTokenDuration = input.SessionDuration
 		input.Config.AssumeRoleDuration = input.SessionDuration
 		input.Config.SSOUseStdout = input.UseStdout
+		input.ShowHelpMessages = !a.Debug && input.Command == "" && isATerminal() && os.Getenv("AWS_VAULT_DISABLE_HELP_MESSAGE") != "1"
 
 		f, err := a.AwsConfigFile()
 		if err != nil {
@@ -129,56 +134,110 @@ func ConfigureExecCommand(app *kingpin.Application, a *AwsVault) {
 			return err
 		}
 
-		err = ExecCommand(input, f, keyring)
+		exitcode := 0
+		if input.JSONDeprecated {
+			exportCommandInput := ExportCommandInput{
+				ProfileName:     input.ProfileName,
+				Format:          "json",
+				Config:          input.Config,
+				SessionDuration: input.SessionDuration,
+				NoSession:       input.NoSession,
+			}
+
+			err = ExportCommand(exportCommandInput, f, keyring)
+		} else {
+			exitcode, err = ExecCommand(input, f, keyring)
+		}
+
 		app.FatalIfError(err, "exec")
+
+		// override exit code if not err
+		os.Exit(exitcode)
+
 		return nil
 	})
 }
 
-func ExecCommand(input ExecCommandInput, f *vault.ConfigFile, keyring keyring.Keyring) error {
+func ExecCommand(input ExecCommandInput, f *vault.ConfigFile, keyring keyring.Keyring) (exitcode int, err error) {
 	if os.Getenv("AWS_VAULT") != "" {
-		return fmt.Errorf("aws-vault sessions should be nested with care, unset AWS_VAULT to force")
+		return 0, fmt.Errorf("running in an existing aws-vault subshell; 'exit' from the subshell or unset AWS_VAULT to force")
 	}
 
-	err := input.validate()
+	if err := input.validate(); err != nil {
+		return 0, err
+	}
+
+	config, err := vault.NewConfigLoader(input.Config, f, input.ProfileName).GetProfileConfig(input.ProfileName)
 	if err != nil {
-		return err
+		return 0, fmt.Errorf("Error loading config: %w", err)
 	}
 
-	vault.UseSession = !input.NoSession
-
-	configLoader := vault.ConfigLoader{
-		File:          f,
-		BaseConfig:    input.Config,
-		ActiveProfile: input.ProfileName,
-	}
-	config, err := configLoader.LoadFromProfile(input.ProfileName)
+	credsProvider, err := vault.NewTempCredentialsProvider(config, &vault.CredentialKeyring{Keyring: keyring}, input.NoSession)
 	if err != nil {
-		return fmt.Errorf("Error loading config: %w", err)
+		return 0, fmt.Errorf("Error getting temporary credentials: %w", err)
 	}
 
-	ckr := &vault.CredentialKeyring{Keyring: keyring}
-	credsProvider, err := vault.NewTempCredentialsProvider(config, ckr)
-	if err != nil {
-		return fmt.Errorf("Error getting temporary credentials: %w", err)
+	subshellHelp := ""
+	if input.Command == "" {
+		input.Command = getDefaultShell()
+		subshellHelp = fmt.Sprintf("Starting subshell %s, use `exit` to exit the subshell", input.Command)
 	}
 
+	cmdEnv := createEnv(input.ProfileName, config.Region)
+
 	if input.StartEc2Server {
-		return execEc2Server(input, config, credsProvider)
+		if server.IsProxyRunning() {
+			return 0, fmt.Errorf("Another process is already bound to 169.254.169.254:80")
+		}
+
+		printHelpMessage("Warning: Starting an EC2 credential server on 169.254.169.254:80; AWS credentials will be accessible to any process while it is running", input.ShowHelpMessages)
+		if err := server.StartEc2EndpointProxyServerProcess(); err != nil {
+			return 0, err
+		}
+		defer server.StopProxy()
+
+		if err = server.StartEc2CredentialsServer(context.TODO(), credsProvider, config.Region); err != nil {
+			return 0, fmt.Errorf("Failed to start credential server: %w", err)
+		}
+		printHelpMessage(subshellHelp, input.ShowHelpMessages)
+	} else if input.StartEcsServer {
+		printHelpMessage("Starting an ECS credential server; your app's AWS sdk must support AWS_CONTAINER_CREDENTIALS_FULL_URI.", input.ShowHelpMessages)
+		if err = startEcsServerAndSetEnv(credsProvider, config, input.Lazy, &cmdEnv); err != nil {
+			return 0, err
+		}
+		printHelpMessage(subshellHelp, input.ShowHelpMessages)
+	} else {
+		if err = addCredsToEnv(credsProvider, input.ProfileName, &cmdEnv); err != nil {
+			return 0, err
+		}
+		printHelpMessage(subshellHelp, input.ShowHelpMessages)
+
+		err = doExecSyscall(input.Command, input.Args, cmdEnv) // will not return if exec syscall succeeds
+		if err != nil {
+			log.Println("Error doing execve syscall:", err.Error())
+			log.Println("Falling back to running a subprocess")
+		}
 	}
 
-	if input.StartEcsServer {
-		return execEcsServer(input, config, credsProvider)
-	}
-
-	if input.CredentialHelper {
-		return execCredentialHelper(input, credsProvider)
-	}
-
-	return execEnvironment(input, config, credsProvider)
+	return runSubProcess(input.Command, input.Args, cmdEnv)
 }
 
-func updateEnvForAwsVault(env environ, profileName string, region string) environ {
+func printHelpMessage(helpMsg string, showHelpMessages bool) {
+	if helpMsg != "" {
+		if showHelpMessages {
+			printToStderr(helpMsg)
+		} else {
+			log.Println(helpMsg)
+		}
+	}
+}
+
+func printToStderr(helpMsg string) {
+	fmt.Fprint(os.Stderr, helpMsg, "\n")
+}
+
+func createEnv(profileName string, region string) environ {
+	env := environ(os.Environ())
 	env.Unset("AWS_ACCESS_KEY_ID")
 	env.Unset("AWS_SECRET_ACCESS_KEY")
 	env.Unset("AWS_SESSION_TOKEN")
@@ -191,27 +250,18 @@ func updateEnvForAwsVault(env environ, profileName string, region string) enviro
 	env.Set("AWS_VAULT", profileName)
 
 	if region != "" {
-		log.Printf("Setting subprocess env: AWS_DEFAULT_REGION=%s, AWS_REGION=%s", region, region)
-		env.Set("AWS_DEFAULT_REGION", region)
+		// AWS_REGION is used by most SDKs. But boto3 (Python SDK) uses AWS_DEFAULT_REGION
+		// See https://docs.aws.amazon.com/sdkref/latest/guide/feature-region.html
+		log.Printf("Setting subprocess env: AWS_REGION=%s, AWS_DEFAULT_REGION=%s", region, region)
 		env.Set("AWS_REGION", region)
+		env.Set("AWS_DEFAULT_REGION", region)
 	}
 
 	return env
 }
 
-func execEc2Server(input ExecCommandInput, config *vault.Config, credsProvider aws.CredentialsProvider) error {
-	if err := server.StartEc2CredentialsServer(context.TODO(), credsProvider, config.Region); err != nil {
-		return fmt.Errorf("Failed to start credential server: %w", err)
-	}
-
-	env := environ(os.Environ())
-	env = updateEnvForAwsVault(env, input.ProfileName, config.Region)
-
-	return execCmd(input.Command, input.Args, env)
-}
-
-func execEcsServer(input ExecCommandInput, config *vault.Config, credsProvider aws.CredentialsProvider) error {
-	ecsServer, err := server.NewEcsServer(context.TODO(), credsProvider, config, "", 0, input.Lazy)
+func startEcsServerAndSetEnv(credsProvider aws.CredentialsProvider, config *vault.ProfileConfig, lazy bool, cmdEnv *environ) error {
+	ecsServer, err := server.NewEcsServer(context.TODO(), credsProvider, config, "", 0, lazy)
 	if err != nil {
 		return err
 	}
@@ -223,79 +273,32 @@ func execEcsServer(input ExecCommandInput, config *vault.Config, credsProvider a
 	}()
 
 	log.Println("Setting subprocess env AWS_CONTAINER_CREDENTIALS_FULL_URI, AWS_CONTAINER_AUTHORIZATION_TOKEN")
-	env := environ(os.Environ())
-	env = updateEnvForAwsVault(env, input.ProfileName, config.Region)
-	env.Set("AWS_CONTAINER_CREDENTIALS_FULL_URI", ecsServer.BaseURL())
-	env.Set("AWS_CONTAINER_AUTHORIZATION_TOKEN", ecsServer.AuthToken())
-
-	return execCmd(input.Command, input.Args, env)
-}
-
-func execCredentialHelper(input ExecCommandInput, credsProvider aws.CredentialsProvider) error {
-	// AwsCredentialHelperData is metadata for AWS CLI credential process
-	// See https://docs.aws.amazon.com/cli/latest/topic/config-vars.html#sourcing-credentials-from-external-processes
-	type AwsCredentialHelperData struct {
-		Version         int    `json:"Version"`
-		AccessKeyID     string `json:"AccessKeyId"`
-		SecretAccessKey string `json:"SecretAccessKey"`
-		SessionToken    string `json:"SessionToken,omitempty"`
-		Expiration      string `json:"Expiration,omitempty"`
-	}
-
-	creds, err := credsProvider.Retrieve(context.TODO())
-	if err != nil {
-		return fmt.Errorf("Failed to get credentials for %s: %w", input.ProfileName, err)
-	}
-
-	credentialData := AwsCredentialHelperData{
-		Version:         1,
-		AccessKeyID:     creds.AccessKeyID,
-		SecretAccessKey: creds.SecretAccessKey,
-		SessionToken:    creds.SessionToken,
-	}
-
-	if creds.CanExpire {
-		credentialData.Expiration = iso8601.Format(creds.Expires)
-	}
-
-	json, err := json.Marshal(&credentialData)
-	if err != nil {
-		return fmt.Errorf("Error creating credential json: %w", err)
-	}
-
-	fmt.Print(string(json))
+	cmdEnv.Set("AWS_CONTAINER_CREDENTIALS_FULL_URI", ecsServer.BaseURL())
+	cmdEnv.Set("AWS_CONTAINER_AUTHORIZATION_TOKEN", ecsServer.AuthToken())
 
 	return nil
 }
 
-func execEnvironment(input ExecCommandInput, config *vault.Config, credsProvider aws.CredentialsProvider) error {
+func addCredsToEnv(credsProvider aws.CredentialsProvider, profileName string, cmdEnv *environ) error {
 	creds, err := credsProvider.Retrieve(context.TODO())
 	if err != nil {
-		return fmt.Errorf("Failed to get credentials for %s: %w", input.ProfileName, err)
+		return fmt.Errorf("Failed to get credentials for %s: %w", profileName, err)
 	}
 
-	env := environ(os.Environ())
-	env = updateEnvForAwsVault(env, input.ProfileName, config.Region)
-
 	log.Println("Setting subprocess env: AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY")
-	env.Set("AWS_ACCESS_KEY_ID", creds.AccessKeyID)
-	env.Set("AWS_SECRET_ACCESS_KEY", creds.SecretAccessKey)
+	cmdEnv.Set("AWS_ACCESS_KEY_ID", creds.AccessKeyID)
+	cmdEnv.Set("AWS_SECRET_ACCESS_KEY", creds.SecretAccessKey)
 
 	if creds.SessionToken != "" {
-		log.Println("Setting subprocess env: AWS_SESSION_TOKEN, AWS_SECURITY_TOKEN")
-		env.Set("AWS_SESSION_TOKEN", creds.SessionToken)
-		env.Set("AWS_SECURITY_TOKEN", creds.SessionToken)
+		log.Println("Setting subprocess env: AWS_SESSION_TOKEN")
+		cmdEnv.Set("AWS_SESSION_TOKEN", creds.SessionToken)
 	}
 	if creds.CanExpire {
-		log.Println("Setting subprocess env: AWS_SESSION_EXPIRATION")
-		env.Set("AWS_SESSION_EXPIRATION", iso8601.Format(creds.Expires))
+		log.Println("Setting subprocess env: AWS_CREDENTIAL_EXPIRATION")
+		cmdEnv.Set("AWS_CREDENTIAL_EXPIRATION", iso8601.Format(creds.Expires))
 	}
 
-	if !supportsExecSyscall() {
-		return execCmd(input.Command, input.Args, env)
-	}
-
-	return execSyscall(input.Command, input.Args, env)
+	return nil
 }
 
 // environ is a slice of strings representing the environment, in the form "key=value".
@@ -318,8 +321,20 @@ func (e *environ) Set(key, val string) {
 	*e = append(*e, key+"="+val)
 }
 
-func execCmd(command string, args []string, env []string) error {
-	log.Printf("Starting child process: %s %s", command, strings.Join(args, " "))
+func getDefaultShell() string {
+	command := os.Getenv("SHELL")
+	if command == "" {
+		if runtime.GOOS == "windows" {
+			command = "cmd.exe"
+		} else {
+			command = "/bin/sh"
+		}
+	}
+	return command
+}
+
+func runSubProcess(command string, args []string, env []string) (int, error) {
+	log.Printf("Starting a subprocess: %s %s", command, strings.Join(args, " "))
 
 	cmd := osexec.Command(command, args...)
 	cmd.Stdin = os.Stdin
@@ -331,9 +346,10 @@ func execCmd(command string, args []string, env []string) error {
 	signal.Notify(sigChan)
 
 	if err := cmd.Start(); err != nil {
-		return err
+		return 0, err
 	}
 
+	// proxy signals to process
 	go func() {
 		for {
 			sig := <-sigChan
@@ -343,19 +359,15 @@ func execCmd(command string, args []string, env []string) error {
 
 	if err := cmd.Wait(); err != nil {
 		_ = cmd.Process.Signal(os.Kill)
-		return fmt.Errorf("Failed to wait for command termination: %v", err)
+		return 0, fmt.Errorf("Failed to wait for command termination: %v", err)
 	}
 
 	waitStatus := cmd.ProcessState.Sys().(syscall.WaitStatus)
-	os.Exit(waitStatus.ExitStatus())
-	return nil
+
+	return waitStatus.ExitStatus(), nil
 }
 
-func supportsExecSyscall() bool {
-	return runtime.GOOS == "linux" || runtime.GOOS == "darwin" || runtime.GOOS == "freebsd" || runtime.GOOS == "openbsd"
-}
-
-func execSyscall(command string, args []string, env []string) error {
+func doExecSyscall(command string, args []string, env []string) error {
 	log.Printf("Exec command %s %s", command, strings.Join(args, " "))
 
 	argv0, err := osexec.LookPath(command)
