@@ -11,10 +11,12 @@ import (
 	"strings"
 	"time"
 
-	"github.com/99designs/aws-vault/v6/vault"
+	"github.com/99designs/aws-vault/v7/vault"
 	"github.com/99designs/keyring"
 	"github.com/alecthomas/kingpin"
 	"github.com/aws/aws-sdk-go-v2/aws"
+	awsconfig "github.com/aws/aws-sdk-go-v2/config"
+	"github.com/aws/aws-sdk-go-v2/credentials"
 	"github.com/skratchdot/open-golang/open"
 )
 
@@ -22,7 +24,7 @@ type LoginCommandInput struct {
 	ProfileName     string
 	UseStdout       bool
 	Path            string
-	Config          vault.Config
+	Config          vault.ProfileConfig
 	SessionDuration time.Duration
 	NoSession       bool
 }
@@ -30,7 +32,7 @@ type LoginCommandInput struct {
 func ConfigureLoginCommand(app *kingpin.Application, a *AwsVault) {
 	input := LoginCommandInput{}
 
-	cmd := app.Command("login", "Generate a login link for the AWS Console")
+	cmd := app.Command("login", "Generate a login link for the AWS Console.")
 
 	cmd.Flag("duration", "Duration of the assume-role or federated session. Defaults to 1h").
 		Short('d').
@@ -59,7 +61,7 @@ func ConfigureLoginCommand(app *kingpin.Application, a *AwsVault) {
 		StringVar(&input.ProfileName)
 
 	cmd.Action(func(c *kingpin.ParseContext) (err error) {
-		input.Config.MfaPromptMethod = a.PromptDriver
+		input.Config.MfaPromptMethod = a.PromptDriver(false)
 		input.Config.NonChainedGetSessionTokenDuration = input.SessionDuration
 		input.Config.AssumeRoleDuration = input.SessionDuration
 		input.Config.GetFederationTokenDuration = input.SessionDuration
@@ -79,14 +81,7 @@ func ConfigureLoginCommand(app *kingpin.Application, a *AwsVault) {
 }
 
 func LoginCommand(input LoginCommandInput, f *vault.ConfigFile, keyring keyring.Keyring) error {
-	vault.UseSession = !input.NoSession
-
-	configLoader := vault.ConfigLoader{
-		File:          f,
-		BaseConfig:    input.Config,
-		ActiveProfile: input.ProfileName,
-	}
-	config, err := configLoader.LoadFromProfile(input.ProfileName)
+	config, err := vault.NewConfigLoader(input.Config, f, input.ProfileName).GetProfileConfig(input.ProfileName)
 	if err != nil {
 		return fmt.Errorf("Error loading config: %w", err)
 	}
@@ -95,13 +90,23 @@ func LoginCommand(input LoginCommandInput, f *vault.ConfigFile, keyring keyring.
 
 	if input.ProfileName == "" {
 		// When no profile is specified, source credentials from the environment
-		credsProvider = vault.NewEnvironmentCredentialsProvider()
+		configFromEnv, err := awsconfig.NewEnvConfig()
+		if err != nil {
+			return fmt.Errorf("unable to authenticate to AWS through your environment variables: %w", err)
+		}
+		credsProvider = credentials.StaticCredentialsProvider{Value: configFromEnv.Credentials}
+		if configFromEnv.Credentials.SessionToken == "" {
+			credsProvider, err = vault.NewFederationTokenProvider(context.TODO(), credsProvider, config)
+			if err != nil {
+				return err
+			}
+		}
 	} else {
 		// Use a profile from the AWS config file
 		ckr := &vault.CredentialKeyring{Keyring: keyring}
-		if config.HasRole() || config.HasSSOStartURL() {
+		if config.HasRole() || config.HasSSOStartURL() || config.HasCredentialProcess() || config.HasWebIdentity() {
 			// If AssumeRole or sso.GetRoleCredentials isn't used, GetFederationToken has to be used for IAM credentials
-			credsProvider, err = vault.NewTempCredentialsProvider(config, ckr)
+			credsProvider, err = vault.NewTempCredentialsProvider(config, ckr, input.NoSession)
 		} else {
 			credsProvider, err = vault.NewFederationTokenCredentialsProvider(context.TODO(), input.ProfileName, ckr, config)
 		}
@@ -177,8 +182,7 @@ func LoginCommand(input LoginCommandInput, f *vault.ConfigFile, keyring keyring.
 	if input.UseStdout {
 		fmt.Println(loginURL)
 	} else if err = open.Run(loginURL); err != nil {
-		log.Println(err)
-		fmt.Println(loginURL)
+		return fmt.Errorf("Failed to open %s: %w", loginURL, err)
 	}
 
 	return nil
