@@ -7,7 +7,7 @@ import (
 	"reflect"
 	"testing"
 
-	"github.com/99designs/aws-vault/v6/vault"
+	"github.com/99designs/aws-vault/v7/vault"
 	"github.com/google/go-cmp/cmp"
 )
 
@@ -38,6 +38,16 @@ region=us-east-1
 
 [profile testincludeprofile2]
 include_profile=testincludeprofile1
+
+[profile with-sso-session]
+sso_session = moon-sso
+sso_account_id=123456
+region = moon-1 # Different from sso region
+
+[sso-session moon-sso]
+sso_start_url = https://d-123456789.example.com/start
+sso_region = moon-2  # Different from profile region
+sso_registration_scopes = sso:account:access
 `)
 
 var nestedConfig = []byte(`[default]
@@ -160,6 +170,7 @@ func TestProfilesFromConfig(t *testing.T) {
 		{Name: "withMFA", MfaSerial: "arn:aws:iam::1234513441:mfa/blah", RoleARN: "arn:aws:iam::4451234513441615400570:role/aws_admin", Region: "us-east-1", DurationSeconds: 1200, SourceProfile: "user2", STSRegionalEndpoints: "legacy"},
 		{Name: "testincludeprofile1", Region: "us-east-1"},
 		{Name: "testincludeprofile2", IncludeProfile: "testincludeprofile1"},
+		{Name: "with-sso-session", SSOSession: "moon-sso", Region: "moon-1", SSOAccountID: "123456"},
 	}
 	actual := cfg.ProfileSections()
 
@@ -194,6 +205,7 @@ func TestAddProfileToExistingConfig(t *testing.T) {
 		{Name: "withMFA", MfaSerial: "arn:aws:iam::1234513441:mfa/blah", RoleARN: "arn:aws:iam::4451234513441615400570:role/aws_admin", Region: "us-east-1", DurationSeconds: 1200, SourceProfile: "user2", STSRegionalEndpoints: "legacy"},
 		{Name: "testincludeprofile1", Region: "us-east-1"},
 		{Name: "testincludeprofile2", IncludeProfile: "testincludeprofile1"},
+		{Name: "with-sso-session", SSOSession: "moon-sso", Region: "moon-1", SSOAccountID: "123456"},
 		{Name: "llamas", MfaSerial: "testserial", Region: "us-east-1", SourceProfile: "default"},
 	}
 	actual := cfg.ProfileSections()
@@ -242,7 +254,7 @@ func TestIncludeProfile(t *testing.T) {
 	}
 
 	configLoader := &vault.ConfigLoader{File: configFile}
-	config, err := configLoader.LoadFromProfile("testincludeprofile2")
+	config, err := configLoader.GetProfileConfig("testincludeprofile2")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -252,6 +264,36 @@ func TestIncludeProfile(t *testing.T) {
 	}
 }
 
+func TestIncludeSsoSession(t *testing.T) {
+	f := newConfigFile(t, exampleConfig)
+	defer os.Remove(f)
+
+	configFile, err := vault.LoadConfig(f)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	configLoader := &vault.ConfigLoader{File: configFile}
+	config, err := configLoader.GetProfileConfig("with-sso-session")
+	if err != nil {
+		t.Fatalf("Should have found a profile: %v", err)
+	}
+
+	if config.Region != "moon-1" { // Test not the same as SSO region
+		t.Fatalf("Expected region %q, got %q", "moon-1", config.Region)
+	}
+
+	ssoStartURL := "https://d-123456789.example.com/start"
+	if config.SSOStartURL != ssoStartURL {
+		t.Fatalf("Expected sso_start_url %q, got %q", ssoStartURL, config.Region)
+	}
+
+	if config.SSORegion != "moon-2" { // Test not the same as profile region
+		t.Fatalf("Expected sso_region %q, got %q", "moon-2", config.Region)
+	}
+	// Not checking sso_registration_scopes as it seems to be unused by aws-cli.
+}
+
 func TestProfileIsEmpty(t *testing.T) {
 	p := vault.ProfileSection{Name: "foo"}
 	if !p.IsEmpty() {
@@ -327,7 +369,7 @@ source_profile=foo
 	}
 
 	configLoader := &vault.ConfigLoader{File: configFile}
-	config, err := configLoader.LoadFromProfile("foo")
+	config, err := configLoader.GetProfileConfig("foo")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -364,7 +406,7 @@ source_profile=root
 	}
 
 	configLoader := &vault.ConfigLoader{File: configFile}
-	config, err := configLoader.LoadFromProfile("foo")
+	config, err := configLoader.GetProfileConfig("foo")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -399,7 +441,7 @@ func TestSetSessionTags(t *testing.T) {
 	}
 
 	for _, tc := range testCases {
-		config := vault.Config{}
+		config := vault.ProfileConfig{}
 		err := config.SetSessionTags(tc.stringValue)
 		if tc.ok {
 			if err != nil {
@@ -431,7 +473,7 @@ func TestSetTransitiveSessionTags(t *testing.T) {
 	}
 
 	for _, tc := range testCases {
-		config := vault.Config{}
+		config := vault.ProfileConfig{}
 		config.SetTransitiveSessionTags(tc.stringValue)
 		if !reflect.DeepEqual(tc.expected, config.TransitiveSessionTags) {
 			t.Fatalf("Expected TransitiveSessionTags: %+v, got %+v", tc.expected, config.TransitiveSessionTags)
@@ -454,7 +496,7 @@ transitive_session_tags = tagOne ,tagTwo,tagThree
 		t.Fatal(err)
 	}
 	configLoader := &vault.ConfigLoader{File: configFile, ActiveProfile: "tagged"}
-	config, err := configLoader.LoadFromProfile("tagged")
+	config, err := configLoader.GetProfileConfig("tagged")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -491,7 +533,7 @@ transitive_session_tags = tagOne ,tagTwo,tagThree
 		t.Fatal(err)
 	}
 	configLoader := &vault.ConfigLoader{File: configFile, ActiveProfile: "tagged"}
-	config, err := configLoader.LoadFromProfile("tagged")
+	config, err := configLoader.GetProfileConfig("tagged")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -536,7 +578,7 @@ source_profile = interim
 		t.Fatal(err)
 	}
 	configLoader := &vault.ConfigLoader{File: configFile, ActiveProfile: "target"}
-	config, err := configLoader.LoadFromProfile("target")
+	config, err := configLoader.GetProfileConfig("target")
 	if err != nil {
 		t.Fatalf("Should have found a profile: %v", err)
 	}
@@ -574,3 +616,39 @@ source_profile = interim
 		t.Fatalf("Expected transitive_session_tags to be empty, got %+v", baseConfig.TransitiveSessionTags)
 	}
 }
+
+func TestValidConfigValidation(t *testing.T) {
+	f := newConfigFile(t, []byte(`
+[profile foo]
+region         = eu-west-1
+mfa_serial     = arn:aws:iam::9999999999999:mfa/david
+
+[profile foo:staging]
+role_arn       = arn:aws:iam::1111111111111:role/admin
+source_profile = foo
+region         = eu-west-2
+mfa_serial     = arn:aws:iam::9999999999999:mfa/david
+
+[profile foo:production]
+role_arn       = arn:aws:iam::1111111111111:role/admin
+source_profile = foo
+region         = eu-west-2
+mfa_serial     = arn:aws:iam::9999999999999:mfa/david
+credential_process = true
+`))
+	defer os.Remove(f)
+	configFile, _ := vault.LoadConfig(f)
+	configLoader := &vault.ConfigLoader{File: configFile}
+
+	config, _ := configLoader.GetProfileConfig("foo:staging")
+	err := config.Validate()
+	if err != nil {
+		t.Fatalf("Should have validated: %v", err)
+	}
+
+	config, _ = configLoader.GetProfileConfig("foo:production")
+	err = config.Validate()
+	if err == nil {
+		t.Fatalf("Should have failed validation: %v", err)
+	}
+}
