@@ -2,13 +2,11 @@ package vault
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"log"
 	"os"
 	"time"
 
-	"github.com/99designs/aws-vault/v6/prompt"
 	"github.com/99designs/keyring"
 	"github.com/aws/aws-sdk-go-v2/aws"
 	"github.com/aws/aws-sdk-go-v2/service/sso"
@@ -45,44 +43,18 @@ func FormatKeyForDisplay(k string) string {
 	return fmt.Sprintf("****************%s", k[len(k)-4:])
 }
 
-// Mfa contains options for an MFA device
-type Mfa struct {
-	MfaToken        string
-	MfaPromptMethod string
-	MfaSerial       string
-}
-
-// GetMfaToken returns the MFA token
-func (m *Mfa) GetMfaToken() (*string, error) {
-	if m.MfaToken != "" {
-		return aws.String(m.MfaToken), nil
-	}
-
-	if m.MfaPromptMethod != "" {
-		promptFunc := prompt.Method(m.MfaPromptMethod)
-		token, err := promptFunc(m.MfaSerial)
-		return aws.String(token), err
-	}
-
-	return nil, errors.New("No prompt found")
-}
-
 // NewMasterCredentialsProvider creates a provider for the master credentials
 func NewMasterCredentialsProvider(k *CredentialKeyring, credentialsName string) *KeyringProvider {
 	return &KeyringProvider{k, credentialsName}
 }
 
-func NewSessionTokenProvider(credsProvider aws.CredentialsProvider, k keyring.Keyring, config *Config) (aws.CredentialsProvider, error) {
+func NewSessionTokenProvider(credsProvider aws.CredentialsProvider, k keyring.Keyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
 	cfg := NewAwsConfigWithCredsProvider(credsProvider, config.Region, config.STSRegionalEndpoints)
 
 	sessionTokenProvider := &SessionTokenProvider{
 		StsClient: sts.NewFromConfig(cfg),
 		Duration:  config.GetSessionTokenDuration(),
-		Mfa: Mfa{
-			MfaToken:        config.MfaToken,
-			MfaPromptMethod: config.MfaPromptMethod,
-			MfaSerial:       config.MfaSerial,
-		},
+		Mfa:       NewMfa(config),
 	}
 
 	if UseSessionCache {
@@ -102,7 +74,7 @@ func NewSessionTokenProvider(credsProvider aws.CredentialsProvider, k keyring.Ke
 }
 
 // NewAssumeRoleProvider returns a provider that generates credentials using AssumeRole
-func NewAssumeRoleProvider(credsProvider aws.CredentialsProvider, k keyring.Keyring, config *Config) (aws.CredentialsProvider, error) {
+func NewAssumeRoleProvider(credsProvider aws.CredentialsProvider, k keyring.Keyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
 	cfg := NewAwsConfigWithCredsProvider(credsProvider, config.Region, config.STSRegionalEndpoints)
 
 	p := &AssumeRoleProvider{
@@ -114,11 +86,7 @@ func NewAssumeRoleProvider(credsProvider aws.CredentialsProvider, k keyring.Keyr
 		Tags:              config.SessionTags,
 		TransitiveTagKeys: config.TransitiveSessionTags,
 		SourceIdentity:    config.SourceIdentity,
-		Mfa: Mfa{
-			MfaSerial:       config.MfaSerial,
-			MfaToken:        config.MfaToken,
-			MfaPromptMethod: config.MfaPromptMethod,
-		},
+		Mfa:               NewMfa(config),
 	}
 
 	if UseSessionCache && config.MfaSerial != "" {
@@ -139,7 +107,7 @@ func NewAssumeRoleProvider(credsProvider aws.CredentialsProvider, k keyring.Keyr
 
 // NewAssumeRoleWithWebIdentityProvider returns a provider that generates
 // credentials using AssumeRoleWithWebIdentity
-func NewAssumeRoleWithWebIdentityProvider(k keyring.Keyring, config *Config) (aws.CredentialsProvider, error) {
+func NewAssumeRoleWithWebIdentityProvider(k keyring.Keyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
 	cfg := NewAwsConfig(config.Region, config.STSRegionalEndpoints)
 
 	p := &AssumeRoleWithWebIdentityProvider{
@@ -167,7 +135,7 @@ func NewAssumeRoleWithWebIdentityProvider(k keyring.Keyring, config *Config) (aw
 }
 
 // NewSSORoleCredentialsProvider creates a provider for SSO credentials
-func NewSSORoleCredentialsProvider(k keyring.Keyring, config *Config) (aws.CredentialsProvider, error) {
+func NewSSORoleCredentialsProvider(k keyring.Keyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
 	cfg := NewAwsConfig(config.SSORegion, config.STSRegionalEndpoints)
 
 	ssoRoleCredentialsProvider := &SSORoleCredentialsProvider{
@@ -196,88 +164,40 @@ func NewSSORoleCredentialsProvider(k keyring.Keyring, config *Config) (aws.Crede
 	return ssoRoleCredentialsProvider, nil
 }
 
-type tempCredsCreator struct {
-	keyring    *CredentialKeyring
-	chainedMfa string
+// NewCredentialProcessProvider creates a provider to retrieve credentials from an external
+// executable as described in https://docs.aws.amazon.com/cli/latest/topic/config-vars.html#sourcing-credentials-from-external-processes
+func NewCredentialProcessProvider(k keyring.Keyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
+	credentialProcessProvider := &CredentialProcessProvider{
+		CredentialProcess: config.CredentialProcess,
+	}
+
+	if UseSessionCache {
+		return &CachedSessionProvider{
+			SessionKey: SessionMetadata{
+				Type:        "credential_process",
+				ProfileName: config.ProfileName,
+			},
+			Keyring:         &SessionKeyring{Keyring: k},
+			ExpiryWindow:    defaultExpirationWindow,
+			CredentialsFunc: credentialProcessProvider.callCredentialProcess,
+		}, nil
+	}
+
+	return credentialProcessProvider, nil
 }
 
-func (t *tempCredsCreator) provider(config *Config) (aws.CredentialsProvider, error) {
-	var sourcecredsProvider aws.CredentialsProvider
-
-	hasStoredCredentials, err := t.keyring.Has(config.ProfileName)
-	if err != nil {
-		return nil, err
-	}
-
-	if hasStoredCredentials && config.HasSourceProfile() {
-		return nil, fmt.Errorf("profile %s: have stored credentials but source_profile is defined", config.ProfileName)
-	} else if hasStoredCredentials {
-		log.Printf("profile %s: using stored credentials", config.ProfileName)
-		sourcecredsProvider = NewMasterCredentialsProvider(t.keyring, config.ProfileName)
-	} else if config.HasSourceProfile() {
-		sourcecredsProvider, err = t.provider(config.SourceProfile)
-		if err != nil {
-			return nil, err
-		}
-	} else if config.HasSSOStartURL() {
-		return NewSSORoleCredentialsProvider(t.keyring.Keyring, config)
-	} else if config.HasRole() && (config.HasWebIdentityTokenFile() || config.HasWebIdentityTokenProcess()) {
-		return NewAssumeRoleWithWebIdentityProvider(t.keyring.Keyring, config)
-	} else {
-		return nil, fmt.Errorf("profile %s: credentials missing", config.ProfileName)
-	}
-
-	if hasStoredCredentials || !config.HasRole() {
-		if canUseGetSessionToken, reason := config.CanUseGetSessionToken(); !canUseGetSessionToken {
-			log.Printf("profile %s: skipping GetSessionToken because %s", config.ProfileName, reason)
-			if !config.HasRole() {
-				return sourcecredsProvider, nil
-			}
-		}
-
-		t.chainedMfa = config.MfaSerial
-		log.Printf("profile %s: using GetSessionToken %s", config.ProfileName, mfaDetails(false, config))
-		sourcecredsProvider, err = NewSessionTokenProvider(sourcecredsProvider, t.keyring.Keyring, config)
-		if !config.HasRole() || err != nil {
-			return sourcecredsProvider, err
-		}
-	}
-
-	isMfaChained := config.MfaSerial != "" && config.MfaSerial == t.chainedMfa
-	if isMfaChained {
-		config.MfaSerial = ""
-	}
-
-	log.Printf("profile %s: using AssumeRole %s", config.ProfileName, mfaDetails(isMfaChained, config))
-	return NewAssumeRoleProvider(sourcecredsProvider, t.keyring.Keyring, config)
-}
-
-func mfaDetails(mfaChained bool, config *Config) string {
-	if mfaChained {
-		return "(chained MFA)"
-	}
-	if config.HasMfaSerial() {
-		return "(with MFA)"
-	}
-	return ""
-}
-
-// NewTempCredentialsProvider creates a credential provider for the given config
-func NewTempCredentialsProvider(config *Config, keyring *CredentialKeyring) (aws.CredentialsProvider, error) {
-	t := tempCredsCreator{
-		keyring: keyring,
-	}
-	return t.provider(config)
-}
-
-func NewFederationTokenCredentialsProvider(ctx context.Context, profileName string, k *CredentialKeyring, config *Config) (aws.CredentialsProvider, error) {
+func NewFederationTokenCredentialsProvider(ctx context.Context, profileName string, k *CredentialKeyring, config *ProfileConfig) (aws.CredentialsProvider, error) {
 	credentialsName, err := FindMasterCredentialsNameFor(profileName, k, config)
 	if err != nil {
 		return nil, err
 	}
-
 	masterCreds := NewMasterCredentialsProvider(k, credentialsName)
-	cfg := NewAwsConfigWithCredsProvider(masterCreds, config.Region, config.STSRegionalEndpoints)
+
+	return NewFederationTokenProvider(ctx, masterCreds, config)
+}
+
+func NewFederationTokenProvider(ctx context.Context, credsProvider aws.CredentialsProvider, config *ProfileConfig) (*FederationTokenProvider, error) {
+	cfg := NewAwsConfigWithCredsProvider(credsProvider, config.Region, config.STSRegionalEndpoints)
 
 	currentUsername, err := GetUsernameFromSession(ctx, cfg)
 	if err != nil {
@@ -292,11 +212,7 @@ func NewFederationTokenCredentialsProvider(ctx context.Context, profileName stri
 	}, nil
 }
 
-func NewEnvironmentCredentialsProvider() aws.CredentialsProvider {
-	return &EnvironmentVariablesCredentialsProvider{}
-}
-
-func FindMasterCredentialsNameFor(profileName string, keyring *CredentialKeyring, config *Config) (string, error) {
+func FindMasterCredentialsNameFor(profileName string, keyring *CredentialKeyring, config *ProfileConfig) (string, error) {
 	hasMasterCreds, err := keyring.Has(profileName)
 	if err != nil {
 		return "", err
@@ -312,3 +228,121 @@ func FindMasterCredentialsNameFor(profileName string, keyring *CredentialKeyring
 
 	return FindMasterCredentialsNameFor(config.SourceProfileName, keyring, config)
 }
+
+type tempCredsCreator struct {
+	// DisableSessions will disable the use of GetSessionToken when set to true
+	DisableSessions bool
+	Keyring         *CredentialKeyring
+
+	chainedMfa string
+}
+
+func (t *tempCredsCreator) getSourceCreds(config *ProfileConfig) (sourcecredsProvider aws.CredentialsProvider, err error) {
+	if config.HasSourceProfile() {
+		log.Printf("profile %s: sourcing credentials from profile %s", config.ProfileName, config.SourceProfile.ProfileName)
+		return t.GetProviderForProfile(config.SourceProfile)
+	}
+
+	hasStoredCredentials, err := t.Keyring.Has(config.ProfileName)
+	if err != nil {
+		return nil, err
+	}
+
+	if hasStoredCredentials {
+		log.Printf("profile %s: using stored credentials", config.ProfileName)
+		return NewMasterCredentialsProvider(t.Keyring, config.ProfileName), nil
+	}
+
+	return nil, fmt.Errorf("profile %s: credentials missing", config.ProfileName)
+}
+
+func (t *tempCredsCreator) GetProviderForProfile(config *ProfileConfig) (aws.CredentialsProvider, error) {
+	if err := config.Validate(); err != nil {
+		return nil, err
+	}
+
+	if config.HasSSOStartURL() {
+		log.Printf("profile %s: using SSO role credentials", config.ProfileName)
+		return NewSSORoleCredentialsProvider(t.Keyring.Keyring, config)
+	}
+
+	if config.HasWebIdentity() {
+		log.Printf("profile %s: using web identity", config.ProfileName)
+		return NewAssumeRoleWithWebIdentityProvider(t.Keyring.Keyring, config)
+	}
+
+	if config.HasCredentialProcess() {
+		log.Printf("profile %s: using credential process", config.ProfileName)
+		return NewCredentialProcessProvider(t.Keyring.Keyring, config)
+	}
+
+	sourcecredsProvider, err := t.getSourceCreds(config)
+	if err != nil {
+		return nil, err
+	}
+
+	if config.HasRole() {
+		isMfaChained := config.MfaSerial != "" && config.MfaSerial == t.chainedMfa
+		if isMfaChained {
+			config.MfaSerial = ""
+		}
+		log.Printf("profile %s: using AssumeRole %s", config.ProfileName, mfaDetails(isMfaChained, config))
+		return NewAssumeRoleProvider(sourcecredsProvider, t.Keyring.Keyring, config)
+	}
+
+	canUseGetSessionToken, reason := t.canUseGetSessionToken(config)
+	if canUseGetSessionToken {
+		t.chainedMfa = config.MfaSerial
+		log.Printf("profile %s: using GetSessionToken %s", config.ProfileName, mfaDetails(false, config))
+		return NewSessionTokenProvider(sourcecredsProvider, t.Keyring.Keyring, config)
+	}
+
+	log.Printf("profile %s: skipping GetSessionToken because %s", config.ProfileName, reason)
+	return sourcecredsProvider, nil
+}
+
+// canUseGetSessionToken determines if GetSessionToken should be used, and if not returns a reason
+func (t *tempCredsCreator) canUseGetSessionToken(c *ProfileConfig) (bool, string) {
+	if t.DisableSessions {
+		return false, "sessions are disabled"
+	}
+
+	if c.IsChained() {
+		if !c.ChainedFromProfile.HasMfaSerial() {
+			return false, fmt.Sprintf("profile '%s' has no MFA serial defined", c.ChainedFromProfile.ProfileName)
+		}
+
+		if !c.HasMfaSerial() && c.ChainedFromProfile.HasMfaSerial() {
+			return false, fmt.Sprintf("profile '%s' has no MFA serial defined", c.ProfileName)
+		}
+
+		if c.ChainedFromProfile.MfaSerial != c.MfaSerial {
+			return false, fmt.Sprintf("MFA serial doesn't match profile '%s'", c.ChainedFromProfile.ProfileName)
+		}
+
+		if c.ChainedFromProfile.AssumeRoleDuration > roleChainingMaximumDuration {
+			return false, fmt.Sprintf("duration %s in profile '%s' is greater than the AWS maximum %s for chaining MFA", c.ChainedFromProfile.AssumeRoleDuration, c.ChainedFromProfile.ProfileName, roleChainingMaximumDuration)
+		}
+	}
+
+	return true, ""
+}
+
+func mfaDetails(mfaChained bool, config *ProfileConfig) string {
+	if mfaChained {
+		return "(chained MFA)"
+	}
+	if config.HasMfaSerial() {
+		return "(with MFA)"
+	}
+	return ""
+}
+
+// NewTempCredentialsProvider creates a credential provider for the given config
+func NewTempCredentialsProvider(config *ProfileConfig, keyring *CredentialKeyring, disableSessions bool) (aws.CredentialsProvider, error) {
+	t := tempCredsCreator{
+		Keyring:         keyring,
+		DisableSessions: disableSessions,
+	}
+	return t.GetProviderForProfile(config)
+}
